local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local select = select
local SPELL_POWER_LIGHT_FORCE = SPELL_POWER_LIGHT_FORCE

c.Init(a)

local spellCosts = {}

local function setCost(name, energy, chi)
	spellCosts[s.SpellName(c.GetID(name))] = { Energy = energy, Chi = chi }
end

function a.GetEnergyCost(localName)
	return spellCosts[localName] and spellCosts[localName].Energy
end

function a.GetChiCost(localName)
	return spellCosts[localName] and spellCosts[localName].Chi
end

setCost("Jab", 40, -1)
setCost("Expel Harm", 40, -1)
setCost("Keg Smash", 40, -2)
setCost("Blackout Kick", 0, 2)
setCost("Tiger Palm", 0, 1)
setCost("Rising Sun Kick", 0, 2)
setCost("Fists of Fury", 0, 3)
setCost("Spinning Crane Kick", 40, -1)
setCost("Dizzying Haze", 0, 0)
setCost("Breath of Fire", 0, 2)
setCost("Touch of Death", 0, 3)
setCost("Guard", 0, 2)
setCost("Purifying Brew", 0, 1)

------------------------------------------------------------------------ Common
local function modSpell(spell)
	spell.NoStopChannel = true
	
	local cost = spellCosts[s.SpellName(spell.ID)]
	if cost then
		spell.EvenIfNotUsable = true
		spell.NoPowerCheck = true
		spell.CheckLast = function()
--c.Debug("CheckLast", s.SpellName(spell.ID), a.Power, cost.Energy, a.Chi, cost.Chi)
			return a.Power >= cost.Energy and a.Chi >= cost.Chi
		end
	end
end

local function addSpell(name, tag, attributes)
	modSpell(c.AddSpell(name, tag, attributes))
end

local function addOptionalSpell(name, tag, attributes)
	modSpell(c.AddOptionalSpell(name, tag, attributes))
end

c.AddOptionalSpell("Legacy of the Emperor", nil, {
	NoRangeCheck = 1,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.FIVE_PERCENT_BUFFS)
	end
})

addOptionalSpell("Touch of Death", nil, {
	CheckFirst = function()
		return c.HasBuff("Death Note")
	end
})

addOptionalSpell("Expel Harm", nil, {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
	end
})

c.AddInterrupt("Spear Hand Strike")

-------------------------------------------------------------------- Brewmaster
addOptionalSpell("Elusive Brew", nil, {
	Buff = "Elusive Brew",
	BuffUnit = "player",
	UseBuffID = true,
	RequireBuff = "Elusive Brew Stacker",
	RequireBuffUnit = "player",
	RequireUseBuffID = true,
})

c.AddOptionalSpell("Stance of the Sturdy Ox", nil, {
	Type = "form",
})

addSpell("Blackout Kick", "for Shuffle", {
	CheckFirst = function(z)
		return c.GetBuffDuration("Shuffle") < 2
			and not c.IsAuraPendingFor("Blackout Kick")
	end
})
c.RegisterAura("Blackout Kick", "Shuffle")

addSpell("Keg Smash", nil, {
	Melee = true,
})

addSpell("Keg Smash", "for Dizzying Haze", {
	Melee = true,
	CheckFirst = function()
		return not s.Debuff(c.GetID("Dizzying Haze"), nil, 2 + c.GetBusyTime())
			and not c.IsAuraPendingFor("Keg Smash")
	end
})
c.RegisterAura("Keg Smash", "Dizzying Haze")

addSpell("Jab", "for Brewmaster", {
	CheckFirst = function()
		if s.MaxPower() - a.Power < a.Regen then
			return true -- we will cap within 1 second
		end
		
		if a.Chi == s.MaxPower("player", SPELL_POWER_LIGHT_FORCE) then
			return false -- don't waste chi
		end
		
		-- jab unless we can wait and get better use out of expel harm
		return c.GetCooldown("Expel Harm") > 40 / a.Regen
	end
})

addSpell("Expel Harm", "for Brewmaster", {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
	end
})

addSpell("Dizzying Haze", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return not s.Debuff(c.GetID("Dizzying Haze"), nil, 2 + c.GetBusyTime())
			and not c.IsAuraPendingFor("Dizzying Haze")
	end
})

addOptionalSpell("Purifying Brew", nil, {
	CheckFirst = function()
		return s.Debuff(c.GetID("Moderate Stagger"), "player")
			or s.Debuff(c.GetID("Heavy Stagger"), "player")
			or (s.Debuff(c.GetID("Light Stagger"), "player") and a.Chi > 2)
	end	
})

addOptionalSpell("Guard", nil, {
	CheckFirst = function()
		return a.Chi >= 4 -- leave enough Chi for Blackout Kick
	end
})

addOptionalSpell("Dampen Harm")

c.AddTaunt("Provoke", nil, { NoGCD = true })

-------------------------------------------------------------------- Windwalker
--c.RegisterAura("Tiger Palm", "Tiger Power")

addSpell("Tiger Palm", "for Tiger Power", {
	CheckFirst = function()
		local stack = c.GetBuffStack("Tiger Power")
		local duration = c.GetBuffDuration("Tiger Power")
		if c.IsCasting("Tiger Palm") then
			stack = stack + 1
			duration = 20
		end
		return stack < 3 or duration < 1.5
	end
})

addSpell("Tiger Palm", "under Combo Breaker", {
	CheckFirst = function()
		return c.HasBuff("Combo Breaker: Tiger Palm")
			and not c.IsCasting("Tiger Palm")
	end
})

addSpell("Tiger Palm", "for Tiger Power AoE", {
	CheckFirst = function()
		local stack = c.GetBuffStack("Tiger Power")
		local duration = c.GetBuffDuration("Tiger Power")
		if c.IsCasting("Tiger Palm") then
			stack = stack + 1
			duration = 20
		end
		return stack < 3 or duration < 3.2
	end
})

addSpell("Tiger Palm", "to save Tiger Power AoE", {
	CheckFirst = function()
		local duration = c.GetBuffDuration("Tiger Power")
		return duration > .1 and duration < 3.2
	end
})

addSpell("Rising Sun Kick")

addSpell("Rising Sun Kick", "for Debuff", {
	CheckFirst = function()
		return not c.HasBuff("Rising Sun Kick")
			and not c.IsAuraPendingFor("Rising Sun Kick")
	end
})

addSpell("Rising Sun Kick", "for Debuff AoE", {
	CheckFirst = function()
		return c.GetBuffDuration("Rising Sun Kick") < 3.2
			and not c.IsAuraPendingFor("Rising Sun Kick")
	end
})

addOptionalSpell("Expel Harm", "at Cap", {
	CheckFirst = function()
		return s.HealthPercent("player") < 90
			and s.MaxPower("player") - s.Power("player") < 10
	end
})

addSpell("Tigereye Brew", nil, {
	CheckFirst = function()
		return c.GetBuffStack("Tigereye Brew") == 10
	end
})

addSpell("Fists of Fury", nil, {
	Melee = true,
	CheckFirst = function()
		return c.GetBuffDuration("Tiger Power") > 4
			and a.Regen * 4 < s.MaxPower("player") - a.Power
	end
})

addSpell("Blackout Kick")

addSpell("Blackout Kick", "under Combo Breaker", {
	CheckFirst = function(z)
		return c.HasBuff("Combo Breaker: Blackout Kick")
			and not c.IsCasting("Blackout Kick")
	end
})

addSpell("Jab")

addSpell("Jab", "at Cap", {
	CheckFirst = function()
		return s.MaxPower("player") - a.Power < a.Regen
	end
})

addSpell("Energizing Brew", nil, {
	CheckFirst = function()
		return s.MaxPower("player") - a.Power > 10 + a.Regen
	end
})

c.AddOptionalSpell("Stance of the Fierce Tiger", nil, {
	Type = "form",
})
